import com.smartvault.proto.NodeProto;

import java.awt.*;
import java.net.URI;
import java.nio.file.Paths;
import java.util.Scanner;

public class Program
{
    private static Scanner keyboard = new Scanner(System.in);

    public static void main(String[] args)
    {
        try
        {
            SmartVault smartVault = new SmartVault();
            authenticate(smartVault);

            String localFile = getLocalFile();
            String remoteFolder = getRemoteFolder(smartVault);

            System.out.println(String.format("\nUploading '%s' to '%s'",
                localFile, remoteFolder));
            smartVault.uploadFile(localFile, remoteFolder);
            System.out.println("\nUpload complete");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        System.out.println("\nPress 'Enter' key to exit");
        keyboard.nextLine();
    }

    private static void authenticate(SmartVault smartVault) throws Exception
    {
        // Autonomous authentication
        System.out.print("Enter your email address: ");
        String email = keyboard.nextLine();

        System.out.print("Have you authorized this application? [y/n]: ");
        String response = keyboard.nextLine();

        if (!response.toLowerCase().startsWith("y"))
        {
            String url = smartVault.requestAuthorization(email);
            Desktop.getDesktop().browse(URI.create(url));
            System.out.println("\nPress 'Enter' once you have allowed this application.");
            keyboard.nextLine();
        }

        smartVault.authenticate(email);
    }

    private static String getLocalFile()
    {
        System.out.print("\nEnter the path to a file to upload. " +
            "(Hint: drag a file to the console window): ");
        String localFile = keyboard.nextLine();

        return localFile.replace("\"", ""); // Remove quotes, if supplied
    }

    private static String getRemoteFolder(SmartVault smartVault) throws Exception
    {
        NodeProto.NodeResponse rootNode = smartVault.browsePath("nodes/pth");
        String account = rootNode.getMessage().getChildrenList().get(0).getName();

        return String.format("/nodes/pth/%s/My First Vault/My First folder", account);
    }
}