import com.smartvault.proto.*;
import com.smartvault.rest.*;
import org.apache.commons.io.IOUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openssl.PEMReader;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.StringReader;
import java.nio.file.Paths;
import java.security.KeyPair;
import java.security.Security;

public class SmartVault
{
    private ClientProtocol _protocol;
    private DelegatedProtocol _delegation;

    public SmartVault() throws Exception
    {
        String clientId = "JavaTutorial"; // Change this to your client ID

        String privateKey = "-----BEGIN RSA PRIVATE KEY-----\n" +
                "MIIEpAIBAAKCAQEAxmh2LZYY5SzT+BMmSNoGsjyI7O6fzmozR+mviyOiAIk0bBXa\n" +
                "U/kLGnaTYe4WE8DyQHweBQq1C5ovUA3L5rqKshoxmhrOtJ04YvMXmEaPBxxfxpLH\n" +
                "ogqg+7kkjDMs+Mb+EpYwguHKGXhFF80AWMZmfVVquTCuhG6bXUHFmdcVSg4imiE2\n" +
                "Ah75matYfD/X9aQecLC32gdgUAx7+6JflmQp+xAhr4s+LUORwCLT6v6znzpA89Qy\n" +
                "/8cc8FS3RrOZtxpjPcY9xK60uY94huPXjuW9Yj9nBgYnqE1hNPZGZCKcwxSInEtC\n" +
                "GjsAx+bvRiNYdNwtOZz6z6oAN/QX4mEQkE2wlQIDAQABAoIBAHkpD4n1EAzbXDIw\n" +
                "139DjTVI3ED706EEn345yVZJk0OKhfW/Po1rFwm5AXWbI+Y6PHV+lgJ0MGz+Yggb\n" +
                "9LlYjxwF/OZjmnkXTXi3hGAcLnmkLSShO8HLhdVSkWwAUGU3d644AzXZ1kXJeT/4\n" +
                "k06jEbFV+Q9QGDKNLNk9MgMpIjoO5Na5doiGd6UpIVGkZhNL1MVRa1x2ktCz211a\n" +
                "b9MvsufNB1d7Ady8lY8z794r035ixclCToimMtd4Q5GPjQIVXhSHGjxtMElRv43k\n" +
                "Xp9EIzdLgOkM7L2tOxMIrBU/5aaErCIfqRER/yIBHc3zlPqh2dp5pK92qx6aml9U\n" +
                "S3uUL90CgYEA+ztEVRNu4dMiJUBQjJxfx8OWZhfsEsVR6G4+aictXrXFQIppiXZE\n" +
                "aZrUiGSZm1lnK7/XdbgPowFvBafObe3MtVDEx5xYE9ZYSI5wm4xyqkybLlr2wmQK\n" +
                "mFdn8l26SpWxnhfBp2nuFk8FyJhuq1kLuTG6zq7xfHZTmH1pSR+jO9sCgYEAyiyG\n" +
                "lHRGe7TMqzlJ0KTUwVBK75b31pAG/yLWIzyeoU/Y7cTNakj/SGloX6kx0Kk2v2/L\n" +
                "lsKFab0wMf1XQgX+C+sxvQ+bM8oiVcvQmlfaRLuFYBzdJRd4xhz69Ir4vRaOAY9K\n" +
                "pbSV9tmEs58oRtm+k/Ks6bRT0KRqTjEebCD8KE8CgYEA7Tpdn+glv9/aUkqUxPG0\n" +
                "D7HEhsr4BBGG8m+HSD2OPexuQi9w0Hh4yS+GmPRFh8e4ADu6fFI9kbJ8315BMtRl\n" +
                "hKCr78DV+oVf/8Ng2cCZVe3YasFCWPSWfJusb9PJYTxQ5/OOzULdqvsbfvtRJUOE\n" +
                "Km+m8huHs/sXnNpvXLkfLhkCgYBg4wuvszVf7Vo0KCUuvv9SkRcIULH++lDQDjLO\n" +
                "+isPJUPRVoIWkFr+qb0iMuQzmhQpk/reCeSDsu2Y107szhBcR98zOnpnTFQ+B4m1\n" +
                "nap1S+R0FgEI2IHBd3vwm3TusRNmvfPJC4zemolUx042/fytzBt/eDK+hzyGslVg\n" +
                "/kG5UwKBgQCG4U6kaT3BjF8Rr+S65JfPKEdZfO6pngva8ySMwB5xdzuX37nMqVZr\n" +
                "TGc654yj1FCa80UMwUo2ptUE0oCKh3VdZElinG9+wNpO/3nIS/b404A5EdF8QJ6w\n" +
                "1lHO1FsptGDOpb69WIjqoUzTiMfMEeOCNKlvFI2ZamzDdR6l39J8Cw==\n" +
                "-----END RSA PRIVATE KEY-----\n";

        Security.addProvider(new BouncyCastleProvider());
        StringReader reader = new StringReader(privateKey);
        KeyPair keyPair = (KeyPair) new PEMReader(reader).readObject();

        ServerCfg cfg = new ServerCfg("https://rest.smartvault.com", clientId, keyPair.getPrivate());
        Authentication auth = new Authentication(cfg);
        _protocol = auth.createClientProtocol();
    }

    public String requestAuthorization(String email) throws Exception
    {
        BasicUserInfoProto.BasicUserInfoResponse response = _protocol.statUser(email);
        return response.getMessage().getAuthorizationUri();
    }

    public void authenticate(String email) throws Exception
    {
        _delegation = _protocol.createDelegatedProtocol(email);
    }

    public NodeProto.NodeResponse browsePath(String path) throws Exception
    {
        int children = 1; // Immediate contents only
        return _delegation.navigate(path, children);
    }

    public void uploadFile(String localFile, String remoteFolder) throws Exception
    {
        FileInputStream stream = new FileInputStream(localFile);

        UploadFileProto.UploadFileRequest.Builder builder =
            UploadFileProto.UploadFileRequest.newBuilder();
        builder.setName(Paths.get(localFile).getFileName().toString());

        _delegation.postFile(remoteFolder, builder.build(), stream);

        stream.close();
    }
}